@@ -2401,8 +2401,6 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[]) {
 
 
 	QCommandLineParser parser;
-	parser.addHelpOption();
-	parser.addVersionOption();
         aboutData.setupCommandLine(&parser);
 	parser.process(app);
         aboutData.processCommandLine(&parser);
