@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 class Constants;
 class QButtonGroup;
+class QLabel;
 class KToggleAction;
 class KCalcConstMenu;
 class KCalcStatusBar;
@@ -56,7 +57,7 @@ class KCalcStatusBar;
 #include "ui_constants.h"
 #include "ui_colors.h"
 
-
+#include <array>
 #include <kxmlguiwindow.h>
 
 class General: public QWidget, public Ui::General
@@ -223,6 +224,8 @@ protected Q_SLOTS:
     void slotBitsetChanged(quint64);
     void slotUpdateBitset(const KNumber &);
 
+    void slotBaseModeAmountChanged(KNumber number);
+
 private:
     enum StatusField {
         ShiftField = 0,
@@ -265,6 +268,8 @@ private:
     QList<QAbstractButton*> stat_buttons_;
     QList<QAbstractButton*> const_buttons_;
 
+    std::array<QLabel*, 4> base_conversion_labels_;
+
     KToggleAction *action_bitset_show_;
     KToggleAction *action_constants_show_;
 
