@@ -30,7 +30,7 @@ class KCalcBitset : public QFrame {
 	Q_OBJECT
 	
 public:
-	explicit KCalcBitset(QWidget *parent = 0);
+	explicit KCalcBitset(QWidget *parent = nullptr);
 	quint64 getValue() const;
 
 public Q_SLOTS:
@@ -41,7 +41,7 @@ Q_SIGNALS:
 	void valueChanged(quint64 value);
 
 private:
-	QButtonGroup *bit_button_group_;
+	QButtonGroup * const bit_button_group_;
 	quint64       value_;
 };
 
